{{st}}
mc_gross={{mc_gross}}
invoice=66
settle_amount=289.83
protection_eligibility=Ineligible
payer_id=8MZ9FQTSAMUPJ
tax=0.00
payment_date=04%3A53%3A52+Apr+12%2C+2009+PDT
payment_status=Completed
charset=windows-1252
first_name=Test
mc_fee=6.88
exchange_rate=1.32876
settle_currency=USD
custom={{custom}}
payer_status=verified
business={{business}}
quantity=1
payer_email=buyer_1239119200_per%40yoursite.com
txn_id={{txn_id}}
payment_type=instant
last_name=User
receiver_email={{business}}
payment_fee=
receiver_id=746LDC2EQAP4W
txn_type=web_accept
item_name=Advertising+Campaign%3A+1+Month+%28225.00%29
mc_currency=EUR
item_number=
residence_country=US
handling_amount=0.00
transaction_subject={{custom}}
payment_gross=
shipping=0.00
-