# -*- coding: utf-8 -*-
# Generated by Django 1.9.1 on 2016-02-19 11:37
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pdt', '0006_auto_20160108_1112'),
    ]

    operations = [
        migrations.AlterField(
            model_name='paypalpdt',
            name='txn_id',
            field=models.CharField(blank=True, db_index=True, help_text='PayPal transaction ID.', max_length=255, verbose_name='Transaction ID'),
        ),
        migrations.AlterField(
            model_name='paypalpdt',
            name='txn_type',
            field=models.CharField(blank=True, help_text='PayPal transaction type.', max_length=255, verbose_name='Transaction Type'),
        ),
    ]
