<html>

<head>
    <title></title>
</head>
<body>

<form action="" method="post">
    <table>
        <tr>
            {{ form.as_table }}
            <td colspan="2" align="right"><input type="submit" value="confirm"/></td>
        </tr>
    </table>
</form>

</body>


</html>


