<html>

<head>
    <title></title>
</head>
<body>

<form action="" method="post">
    <table>
        {{ form.as_p }}
        <tr>
            <td colspan="2" align="right">
                <input type="submit"/>
            </td>
        </tr>


    </table>
</form>

</body>


</html>


