# Generated by Django 2.1.3 on 2018-11-28 10:32

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pro', '0002_auto_20141117_1647'),
    ]

    operations = [
        migrations.AlterField(
            model_name='paypalnvp',
            name='flag',
            field=models.BooleanField(blank=True, default=False),
        ),
    ]
